/*		 ITI-FG  Kernel module free public release
 *
 *		     Copyright (C) 1996-2000 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: amvsReg.c,v 7.0 2000/10/25 09:07:57 cvs Exp $
 *
 */

/*
 * Defines:
 */

/*
 * Includes:
 */

#include "itifgOS.h"	/* os specific stuff */
#include "itifgExt.h"	/* offical driver interface */
#include "itifgReg.h"	/* register definitions ICR Regs */
#include "icpciReg.h"	/* register definitions IC-PCI */
#include "amvsReg.h"	/* register definitions AM-VS */
#include "amdigReg.h"	/* register definitions AM-DIG */
#include "ampcvReg.h"	/* register definitions PCVision */
#include "pcdigReg.h"	/* register definitions PC-DIG */
#include "itifgIntD.h"	/* driver internals */
#include "itifgIntF.h"	/* internal functions */

/*
 * Globals:
 */

#if (DEBUG_ITIFG & 132) /* 7 + 2 */
const char  *vs_reg[] = { "modid ",
			  "con1  ",
			  "con2  ",
			  "acqcon",
			  "hoff  ",
			  "hsz   ",
			  "voff  ",
			  "vsz   ",
			  "froff ",
			  "cpcon ",
			  "swtrig",
			  "error ",
			  "error ",
			  "error ",
			  "error ",
			  "error ",
			  "lutadr",
			  "lutdat",
			  "adccon"
			};
#endif

/*
 * Prototypes:
 */

static u_short vs_read_mm (itifg_t*, u_char, u_short, u_short, u_short);
static u_char vs_read_ms (itifg_t*, u_char, u_short, u_char);
static int vs_write_mm (itifg_t*, u_char, u_short, u_short, u_short, u_short);
static int vs_write_ms (itifg_t*, u_char, u_short, u_char, u_char);

/*
 * General Read Handler for AM-VS Registers
 */

/* 16 bit register access with mask and range checking */

static u_short
vs_read_mm (iti, offset, mask, min, max)
     
     itifg_t *iti;	/* number of IC-PCI board attached to the computer */
     u_char  offset;	/* offset to register base address pointer */
     u_short mask;	/* read mask for register */
     u_short min;	/* minimal value for range checking */
     u_short max;	/* maximal value for range checking */

{
  u_short tmp_val = 0;

  tmp_val = iti_readw (iti->v_mod_base + offset);

  if (offset != VS_MODID_OFFSET)
    tmp_val &= VS_MAGIC_MASK;

  tmp_val &= mask;

#if(0)
  if (tmp_val & ~mask) iti->errno = -EITI_MASK;

  if (tmp_val < min || tmp_val > max) iti->errno = -EITI_RANGE;
#endif

  ITI_PRINT7A("vs-readw %s, %#06X &(%#06X) >%#06X <%#06X\n",
	      vs_reg[offset], tmp_val, mask, min, max);

  return (tmp_val);
}

/* 16 bit register 1 (or more) bit access */

static u_char
vs_read_ms (iti, offset, mask, shift)

     itifg_t *iti;	/* number of IC-PCI board attached to the computer */
     u_char  offset;	/* offset to register base address pointer */
     u_short mask;	/* read mask for register */
     u_char  shift;	/* number of bits to shift right */
  
{
  u_short tmp_val = 0;

  tmp_val = iti_readw (iti->v_mod_base + offset);

  if (offset != VS_MODID_OFFSET)
    tmp_val &= VS_MAGIC_MASK;

  tmp_val &= mask;
  tmp_val >>= shift;

  ITI_PRINT7A(" vs-readw %s, %#1X &(%#06X) >>%2d\n",
	       vs_reg[offset], tmp_val, mask, shift);

  return ((u_char)tmp_val);
}
  
/*
 * General Write Handler for AM-VS Registers
 */

/* 16 bit register access with mask and range checking */

static int
vs_write_mm (iti, offset, mask, min, max, value)

     itifg_t *iti;	/* number of IC-PCI board attached to the computer */
     u_char  offset;	/* offset to register base address pointer */
     u_short mask;	/* read mask for register */
     u_short min;	/* minimal value for range checking */
     u_short max;	/* maximal value for range checking */
     u_short value;	/* value to set to */

{
  u_short tmp_val = 0;

  if (value & ~mask) {
    iti->errno = -EITI_MASK;
    return (ERROR);
  }

  if (value < min || value > max) {
    iti->errno = -EITI_RANGE;  
    return (ERROR);
  }

  tmp_val = value & mask;

  ITI_PRINT7A("vs-writew %s, %#06X &(%#06X) >%#06X <%#06X\n",
	       vs_reg[offset], tmp_val, mask, min, max);

  tmp_val |= ~VS_MAGIC_MASK;

  iti_writew (tmp_val, iti->v_mod_base + offset);

  return (OK);
}

/* 16 bit register 1 (or more) bit access */

static int
vs_write_ms (iti, offset, mask, shift, value)

     itifg_t *iti;	/* number of IC-PCI board attached to the computer */
     u_char  offset;	/* offset to register base address pointer */
     u_short mask;	/* read mask for register */
     u_char  shift;	/* number of bits to shift right */
     u_char  value;	/* value to set to */  
  
{
  u_short read_val = 0;
  u_short write_val = 0;
  u_short tmp_val = 0;

  tmp_val = (u_short)value << shift;

  if (tmp_val & ~mask) {
    iti->errno = -EITI_MASK;
    return (ERROR);
  }

  read_val = iti_readw (iti->v_mod_base + offset);

  if (offset != VS_MODID_OFFSET)
    read_val &= VS_MAGIC_MASK;

  read_val &= ~mask;
  write_val = read_val | tmp_val;

  ITI_PRINT7A("vs-writew %s, %#06X->%#06X %#1X &(%#06X) <<%2d\n",
	       vs_reg[offset], read_val, write_val, tmp_val, mask, shift);

  write_val |= ~VS_MAGIC_MASK;

  iti_writew (write_val, iti->v_mod_base + offset);

  return (OK);
}
  
/*
 * Special Read Handler for AM-VS Registers
 */

u_char
vs_get_rev (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_MODID_OFFSET,
		      VS_REV_MASK,
		      VS_REV_SHIFT));
}

u_char
vs_get_tgid (itifg_t *iti) {

  u_char tmp_val = 0;
  
  tmp_val = vs_read_ms (iti,
			VS_MODID_OFFSET,
			VS_TGID_MASK,
			VS_TGID_SHIFT);

#if(0)
  if (tmp_val > 2) iti->errno = -EITI_RANGE;
#endif

  return (tmp_val);
}

u_char
vs_get_tsel (itifg_t *iti) {

  u_char tgid = 0, tsel = 0;
  
  tgid = vs_get_tgid (iti);
  
  tsel = vs_read_ms (iti,
		     VS_CON1_OFFSET,
		     VS_TSEL_MASK,
		     VS_TSEL_SHIFT);

#if(0)
  switch (tgid) {
  case 1:
    if (tsel == 5) iti->errno = -EITI_RANGE;
    break;
  case 0:
    if (tsel == 1 || tsel == 5) iti->errno = -EITI_RANGE;	
    break;
  }
#endif
  
  return (tsel);
}

u_char
vs_get_clksrc (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_CON1_OFFSET,
		      VS_CLKSRC_MASK,
		      VS_CLKSRC_SHIFT));
}

u_char
vs_get_lpfsel (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_CON1_OFFSET,
		      VS_LPFSEL_MASK,
		      VS_LPFSEL_SHIFT));
}

u_char
vs_get_iscale (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_CON1_OFFSET,
		      VS_ISCALE_MASK,
		      VS_ISCALE_SHIFT));
}

u_char
vs_get_smode (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_CON1_OFFSET,
		      VS_SMODE_MASK,
		      VS_SMODE_SHIFT));
}

#if(0)	/* ? not supported ? */
u_char
vs_get_camsel (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_CON1_OFFSET,
		      VS_CAMSEL_MASK,
		      VS_CAMSEL_SHIFT));
}
#endif

u_char
vs_get_tsrc (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_CON2_OFFSET,
		      VS_TSRC_MASK,
		      VS_TSRC_SHIFT));
}

u_char
vs_get_lenpol (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_CON2_OFFSET,
		      VS_LENPOL_MASK,
		      VS_LENPOL_SHIFT));
}

u_char
vs_get_fldpol (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_CON2_OFFSET,
		      VS_FLDPOL_MASK,
		      VS_FLDPOL_SHIFT));
}

u_char
vs_get_fenpol (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_CON2_OFFSET,
		      VS_FENPOL_MASK,
		      VS_FENPOL_SHIFT));
}

u_char
vs_get_pclkpol (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_CON2_OFFSET,
		      VS_PCLKPOL_MASK,
		      VS_PCLKPOL_SHIFT));
}

u_char
vs_get_eclken (itifg_t *iti) {

  u_char tsel = 0, eclken = 0;
  
  tsel = vs_get_tsel (iti);
  
  eclken = vs_read_ms (iti,
		       VS_CON2_OFFSET,
		       VS_ECLKEN_MASK,
		       VS_ECLKEN_SHIFT);
  
#if(0)
  switch (tsel) {
  case 5:
    iti->errno = -EITI_RANGE;
    break;
  case 1:
    if (eclken) iti->errno = -EITI_RANGE;
    break;
  }
#endif

  return (eclken);
}

u_char
vs_get_vbstat (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_CON2_OFFSET,
		      VS_VBSTAT_MASK,
		      VS_VBSTAT_SHIFT));
}

u_char
vs_get_fldsten (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_CON2_OFFSET,
		      VS_FLDSTEN_MASK,
		      VS_FLDSTEN_SHIFT));
}

u_char
vs_get_vsten (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_CON2_OFFSET,
		      VS_VSTEN_MASK,
		      VS_VSTEN_SHIFT));
}

u_char
vs_get_trigen (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_TRIGCON_OFFSET,
		      VS_TRIGEN_MASK,
		      VS_TRIGEN_SHIFT));
}

u_char
vs_get_trigsrc (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_TRIGCON_OFFSET,
		      VS_TRIGSRC_MASK,
		      VS_TRIGSRC_SHIFT));
}

u_char
vs_get_fmode  (itifg_t *iti) {

#if(0)
  if (vs_get_rev (iti) < 1) iti->errno = -EITI_RANGE;
#endif

  return (vs_read_ms (iti,
		      VS_TRIGCON_OFFSET,
		      VS_FMODE_MASK,
		      VS_FMODE_SHIFT));
}

u_char
vs_get_tmode (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_TRIGCON_OFFSET,
		      VS_TMODE_MASK,
		      VS_TMODE_SHIFT));
}

u_char
vs_get_trigcy (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_TRIGCON_OFFSET,
		      VS_TRIGCY_MASK,
		      VS_TRIGCY_SHIFT));
}

u_char
vs_get_strbdly (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_TRIGCON_OFFSET,
		      VS_STRBDLY_MASK,
		      VS_STRBDLY_SHIFT));
}

u_short
vs_get_hoff (itifg_t *iti) {

  return (vs_read_mm (iti,
		      VS_HOFF_OFFSET,
		      VS_HOFF_MASK,
		      VS_HOFF_MIN,
		      VS_HOFF_MAX));
}

u_short
vs_get_hsz (itifg_t *iti) {

  u_short tmp_val = 0; 

  tmp_val = vs_read_mm (iti,
			VS_HSZ_OFFSET,
			VS_HSZ_MASK,
			VS_HSZ_MIN,
			VS_HSZ_MAX);
  
  return ((tmp_val + 1) << 2);
}

u_short
vs_get_voff (itifg_t *iti) {

  return (vs_read_mm (iti,
		      VS_VOFF_OFFSET,
		      VS_VOFF_MASK,
		      VS_VOFF_MIN,
		      VS_VOFF_MAX));
}

u_short
vs_get_vsz (itifg_t *iti) {

  u_short tmp_val = 0; 

  tmp_val = vs_read_mm (iti,
			VS_VSZ_OFFSET,
			VS_VSZ_MASK,
			VS_VSZ_MIN,
			VS_VSZ_MAX);

  return (tmp_val + 1);
}

u_short
vs_get_froff (itifg_t *iti) {

  return (vs_read_mm (iti,
		      VS_FROFF_OFFSET,
		      VS_FROFF_MASK,
		      VS_FROFF_MIN,
		      VS_FROFF_MAX));
}

u_char
vs_get_cppos (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_CPCON_OFFSET,
		      VS_CPPOS_MASK,
		      VS_CPPOS_SHIFT));
}

u_char
vs_get_cpen (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_CPCON_OFFSET,
		      VS_CPEN_MASK,
		      VS_CPEN_SHIFT));
}

u_char
vs_get_cpsrc (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_CPCON_OFFSET,
		      VS_CPSRC_MASK,
		      VS_CPSRC_SHIFT));
}

u_char
vs_get_strbsel (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_CPCON_OFFSET,
		      VS_STRBSEL_MASK,
		      VS_STRBSEL_SHIFT));
}

u_char
vs_get_strbpol (itifg_t *iti) {

  return (vs_read_ms (iti,
		      VS_CPCON_OFFSET,
		      VS_STRBPOL_MASK,
		      VS_STRBPOL_SHIFT));
}
			
u_char
vs_get_cpst (itifg_t *iti) {

#if(0)
  if (vs_get_rev (iti) < 2) iti->errno = -EITI_RANGE;
#endif

  return (vs_read_ms (iti,
		      VS_CPCON_OFFSET,
		      VS_CPST_MASK,
		      VS_CPST_SHIFT));
}

u_char
vs_get_lutdata (itifg_t *iti, struct vs_lut_t *get_lut) {

  int count;

  vs_write_mm (iti,
	       VS_LUTADDR_OFFSET,
	       VS_LUTADDR_MASK,
	       VS_LUTADDR_MIN,
	       VS_LUTADDR_MAX,
	       VS_LUTADDR_MIN);

#if(0)
  if (get_lut == NULL) iti->errno = -EITI_POINTER;
  if (iti->errno < 0) return (ERROR);
#endif

  for (count = VS_LUTADDR_MIN; count <= VS_LUTADDR_MAX; count++) {

    get_lut->lutdata[count] = vs_read_mm (iti,
					  VS_LUTDATA_OFFSET,
					  VS_LUTDATA_MASK,
					  VS_LUTDATA_MIN,
					  VS_LUTDATA_MAX);

#if(0)    
    if (iti->errno < 0) return (ERROR);
#endif

  }
  return (OK);
}

u_short
vs_get_nref (itifg_t *iti) {

  vs_write_mm (iti,
	       VS_LUTADDR_OFFSET,
	       VS_LUTADDR_MASK,
	       VS_LUTADDR_MIN,
	       VS_LUTADDR_MAX,
	       VS_LUTADDR_ADC_NR);

#if(0)  
  if (iti->errno < 0) return (0);
#endif
  
  return (vs_read_mm (iti,
		      VS_NREF_OFFSET,
		      VS_NREF_MASK,
		      VS_NREF_MIN,
		      VS_NREF_MAX));
}

u_short
vs_get_pref (itifg_t *iti) {

  vs_write_mm (iti,
	       VS_LUTADDR_OFFSET,
	       VS_LUTADDR_MASK,
	       VS_LUTADDR_MIN,
	       VS_LUTADDR_MAX,
	       VS_LUTADDR_ADC_PR);

#if(0)  
  if (iti->errno < 0) return (0);
#endif
  
  return (vs_read_mm (iti,
		      VS_PREF_OFFSET,
		      VS_PREF_MASK,
		      VS_PREF_MIN,
		      VS_PREF_MAX));
}
			
/*
 * Special Write Handler for AM-VS Registers
 */

int
vs_set_tsel (itifg_t *iti, u_char value) {

    u_char tgid = 0;

    tgid = vs_get_tgid (iti);

    switch (tgid) {
    case 1:
	if (value == 5) {
	    iti->errno = -EITI_NOSUPPORT;
	    return (ERROR);
	}
	break;
    case 0:
	if (value == 1 || value == 5) {
	    iti->errno = -EITI_NOSUPPORT;
	    return (ERROR);
	}
	break;
    }	

    return (vs_write_ms (iti,
			 VS_CON1_OFFSET,
			 VS_TSEL_MASK,
			 VS_TSEL_SHIFT,
			 value));
}

int
vs_set_clksrc (itifg_t *iti, u_char value) {

  return (vs_write_ms (iti,
		       VS_CON1_OFFSET,
		       VS_CLKSRC_MASK,
		       VS_CLKSRC_SHIFT,
		       value));
}

int
vs_set_lpfsel (itifg_t *iti, u_char value) {

  return (vs_write_ms (iti,
		       VS_CON1_OFFSET,
		       VS_LPFSEL_MASK,
		       VS_LPFSEL_SHIFT,
		       value));
}

int
vs_set_iscale (itifg_t *iti, u_char value) {

  return (vs_write_ms (iti,
		       VS_CON1_OFFSET,
		       VS_ISCALE_MASK,
		       VS_ISCALE_SHIFT,
		       value));
}

int
vs_set_smode (itifg_t *iti, u_char value) {

  return (vs_write_ms (iti,
		       VS_CON1_OFFSET,
		       VS_SMODE_MASK,
		       VS_SMODE_SHIFT,
		       value));
}

int
vs_set_camsel (itifg_t *iti, u_char value) {

  return (vs_write_ms (iti,
		       VS_CON1_OFFSET,
		       VS_CAMSEL_MASK,
		       VS_CAMSEL_SHIFT,
		       value));
}

int
vs_set_tsrc (itifg_t *iti, u_char value) {

  return (vs_write_ms (iti,
		       VS_CON2_OFFSET,
		       VS_TSRC_MASK,
		       VS_TSRC_SHIFT,
		       value));
}

int
vs_set_lenpol (itifg_t *iti, u_char value) {

  return (vs_write_ms (iti,
		       VS_CON2_OFFSET,
		       VS_LENPOL_MASK,
		       VS_LENPOL_SHIFT,
		       value));
}

int
vs_set_fldpol (itifg_t *iti, u_char value) {

  return (vs_write_ms (iti,
		       VS_CON2_OFFSET,
		       VS_FLDPOL_MASK,
		       VS_FLDPOL_SHIFT,
		       value));
}

int
vs_set_fenpol (itifg_t *iti, u_char value) {

  return (vs_write_ms (iti,
		       VS_CON2_OFFSET,
		       VS_FENPOL_MASK,
		       VS_FENPOL_SHIFT,
		       value));
}

int
vs_set_pclkpol (itifg_t *iti, u_char value) {

  return (vs_write_ms (iti,
		       VS_CON2_OFFSET,
		       VS_PCLKPOL_MASK,
		       VS_PCLKPOL_SHIFT,
		       value));
}

int
vs_set_eclken (itifg_t *iti, u_char value) {

  u_char tsel = 0;
  
  tsel = vs_get_tsel (iti);
  
  switch (tsel) {
  case 5:
    iti->errno = -EITI_NOSUPPORT;
    return (ERROR);
  case 1:
    if (value) {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
    break;
  }
  
  return (vs_write_ms (iti,
		       VS_CON2_OFFSET,
		       VS_ECLKEN_MASK,
		       VS_ECLKEN_SHIFT,
		       value));
  
  return (OK);
}

int
vs_set_fldsten (itifg_t *iti, u_char value) {

  return (vs_write_ms (iti,
		       VS_CON2_OFFSET,
		       VS_FLDSTEN_MASK,
		       VS_FLDSTEN_SHIFT,
		       value));
}

int
vs_set_vsten (itifg_t *iti, u_char value) {

  return (vs_write_ms (iti,
		       VS_CON2_OFFSET,
		       VS_VSTEN_MASK,
		       VS_VSTEN_SHIFT,
		       value));
}

int
vs_set_trigen (itifg_t *iti, u_char value) {

  if (value == VS_ENABLE) {
    if (vs_get_trigcy (iti) == VS_IN_TRIGCY) {
      iti->errno = -EITI_BUSY;
      return (ERROR);
    }
  }

  return (vs_write_ms (iti,
		       VS_TRIGCON_OFFSET,
		       VS_TRIGEN_MASK,
		       VS_TRIGEN_SHIFT,
		       value));
}

int
vs_set_trigsrc (itifg_t *iti, u_char value) {

  return (vs_write_ms (iti,
		       VS_TRIGCON_OFFSET,
		       VS_TRIGSRC_MASK,
		       VS_TRIGSRC_SHIFT,
		       value));
}

int
vs_set_fmode (itifg_t *iti, u_char value) {

    if (vs_get_rev (iti) < 1) {
	iti->errno = -EITI_NOSUPPORT;
	return (ERROR);
    }

    return (vs_write_ms (iti,
			 VS_TRIGCON_OFFSET,
			 VS_FMODE_MASK,
			 VS_FMODE_SHIFT,
			 value));
}

int
vs_set_tmode (itifg_t *iti, u_char value) {

  return (vs_write_ms (iti,
		       VS_TRIGCON_OFFSET,
		       VS_TMODE_MASK,
		       VS_TMODE_SHIFT,
		       value));
}

int
vs_set_strbdly (itifg_t *iti, u_char value) {

  if (value > VS_STRBDLY_MAX) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }
  
  return (vs_write_ms (iti,
		       VS_TRIGCON_OFFSET,
		       VS_STRBDLY_MASK,
		       VS_STRBDLY_SHIFT,
		       value));
}

int
vs_set_hoff (itifg_t *iti, u_short value) {

  return (vs_write_mm (iti,
		       VS_HOFF_OFFSET,
		       VS_HOFF_MASK,
		       VS_HOFF_MIN,
		       VS_HOFF_MAX,
		       value));
}

int
vs_set_hsz (itifg_t *iti, u_short value) {

  return (vs_write_mm (iti,
		       VS_HSZ_OFFSET,
		       VS_HSZ_MASK,
		       VS_HSZ_MIN,
		       VS_HSZ_MAX,
		       (value - 1) >> 2));
}

int
vs_set_voff (itifg_t *iti, u_short value) {

  return (vs_write_mm (iti,
		       VS_VOFF_OFFSET,
		       VS_VOFF_MASK,
		       VS_VOFF_MIN,
		       VS_VOFF_MAX,
		       value));
}

int
vs_set_vsz (itifg_t *iti, u_short value) {

  return (vs_write_mm (iti,
		       VS_VSZ_OFFSET,
		       VS_VSZ_MASK,
		       VS_VSZ_MIN,
		       VS_VSZ_MAX,
		       value - 1));
}

int
vs_set_froff (itifg_t *iti, u_short value) {

  return (vs_write_mm (iti,
		       VS_FROFF_OFFSET,
		       VS_FROFF_MASK,
		       VS_FROFF_MIN,
		       VS_FROFF_MAX,
		       value));
}

int
vs_set_cppos (itifg_t *iti, u_char value) {

  if (value > VS_CPPOS_MAX) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }

  return (vs_write_ms (iti,
		       VS_CPCON_OFFSET,
		       VS_CPPOS_MASK,
		       VS_CPPOS_SHIFT,
		       value));
}

int
vs_set_cpen (itifg_t *iti, u_char value) {

  return (vs_write_ms (iti,
		       VS_CPCON_OFFSET,
		       VS_CPEN_MASK,
		       VS_CPEN_SHIFT,
		       value));
}

int
vs_set_cpsrc (itifg_t *iti, u_char value) {

  return (vs_write_ms (iti,
		       VS_CPCON_OFFSET,
		       VS_CPSRC_MASK,
		       VS_CPSRC_SHIFT,
		       value));
}

int
vs_set_strbsel (itifg_t *iti, u_char value) {

  return (vs_write_ms (iti,
		       VS_CPCON_OFFSET,
		       VS_STRBSEL_MASK,
		       VS_STRBSEL_SHIFT,
		       value));
}

int
vs_set_strbpol (itifg_t *iti, u_char value) {

  return (vs_write_ms (iti,
		       VS_CPCON_OFFSET,
		       VS_STRBPOL_MASK,
		       VS_STRBPOL_SHIFT,
		       value));
}

int
vs_set_cpst (itifg_t *iti, u_char value) {

  if (vs_get_rev (iti) < 2) {
    iti->errno = -EITI_NOSUPPORT;
    return (ERROR);
  }


  return (vs_write_ms (iti,
		       VS_CPCON_OFFSET,
		       VS_CPST_MASK,
		       VS_CPST_SHIFT,
		       value));
}

int
vs_set_lutdata (itifg_t *iti, struct vs_lut_t *set_lut) {

  int count;

  if (set_lut == NULL) {
    iti->errno = -EITI_POINTER;
    return (ERROR);
  }

  if (vs_write_mm (iti,
		   VS_LUTADDR_OFFSET,
		   VS_LUTADDR_MASK,
		   VS_LUTADDR_MIN,
		   VS_LUTADDR_MAX,
		   VS_LUTADDR_MIN) < 0) return (ERROR);
   
  for (count = VS_LUTADDR_MIN; count <= VS_LUTADDR_MAX; count++) {

  if (vs_write_mm (iti,
		   VS_LUTDATA_OFFSET,
		   VS_LUTDATA_MASK,
		   VS_LUTDATA_MIN,
		   VS_LUTDATA_MAX,
		   set_lut->lutdata[count]) < 0) return (ERROR);
  }
  return (OK);
}

int
vs_set_nref (itifg_t *iti, u_short value) {

  if (vs_write_mm (iti,
		   VS_LUTADDR_OFFSET,
		   VS_LUTADDR_MASK,
		   VS_LUTADDR_MIN,
		   VS_LUTADDR_MAX,
		   VS_LUTADDR_ADC_NR) < 0) return (ERROR);

  return (vs_write_mm (iti,
		       VS_NREF_OFFSET,
		       VS_NREF_MASK,
		       VS_NREF_MIN,
		       VS_NREF_MAX,
		       value));
}

int
vs_set_pref (itifg_t *iti, u_short value) {

  if (vs_write_mm (iti,
		   VS_LUTADDR_OFFSET,
		   VS_LUTADDR_MASK,
		   VS_LUTADDR_MIN,
		   VS_LUTADDR_MAX,
		   VS_LUTADDR_ADC_PR) < 0) return (ERROR);
  
  return (vs_write_mm (iti,
		       VS_PREF_OFFSET,
		       VS_PREF_MASK,
		       VS_PREF_MIN,
		       VS_PREF_MAX,
		       value));
}

/*
 * Special Initialization Handler for AM-VS Registers
 */

int
vs_init_con1 (itifg_t *iti) {

  return (vs_write_mm (iti,
		       VS_CON1_OFFSET,
		       VS_NO_MASK,
		       VS_NO_MIN,
		       VS_NO_MAX,
		       VS_RES_INIT));
}

int
vs_init_con2 (itifg_t *iti) {
    
  return (vs_write_mm (iti,
		       VS_CON2_OFFSET,
		       VS_NO_MASK,
		       VS_NO_MIN,
		       VS_NO_MAX,
		       VS_RES_INIT));
}

int
vs_init_trigcon (itifg_t *iti) {
    
    return (vs_write_mm (iti,
			 VS_TRIGCON_OFFSET,
			 VS_NO_MASK,
			 VS_NO_MIN,
			 VS_NO_MAX,
			 VS_RES_INIT));
}

int
vs_init_cpcon (itifg_t *iti) {
    
    return (vs_write_mm (iti,
			 VS_CPCON_OFFSET,
			 VS_NO_MASK,
			 VS_NO_MIN,
			 VS_NO_MAX,
			 VS_RES_INIT));
}

int
vs_init_adccon (itifg_t *iti) {
    
  if (vs_write_mm (iti,
		   VS_LUTADDR_OFFSET,
		   VS_LUTADDR_MASK,
		   VS_LUTADDR_MIN,
		   VS_LUTADDR_MAX,
		   VS_LUTADDR_ADC_CR) < 0) return (ERROR);

  return (vs_write_mm (iti,
		       VS_ADCCON_OFFSET,
		       VS_NO_MASK,
		       VS_NO_MIN,
		       VS_NO_MAX,
		       VS_ADCCON_INIT));
}

/* 
 * Miscellanous functions
 */
 
/* trigger per software */

int
vs_sw_trig (itifg_t *iti) {

  return (vs_write_mm (iti,
		       VS_SWTRIG_OFFSET,
		       VS_SWTRIG_MASK,
		       VS_SWTRIG_MIN,
		       VS_SWTRIG_MAX,
		       VS_SWTRIG));
}

/* linearize lookup table */

int
vs_lin_lut (itifg_t *iti) {

  int count;

  if (vs_write_mm (iti,
		   VS_LUTADDR_OFFSET,
		   VS_LUTADDR_MASK,
		   VS_LUTADDR_MIN,
		   VS_LUTADDR_MAX,
		   VS_LUTADDR_MIN) < 0) return (ERROR);

  for (count = VS_LUTADDR_MIN; count <= VS_LUTADDR_MAX; count++) {

    if (vs_write_mm (iti,
		     VS_LUTDATA_OFFSET,
		     VS_LUTDATA_MASK,
		     VS_LUTDATA_MIN,
		     VS_LUTDATA_MAX,
		     count) < 0) return (ERROR);
  }
  return (OK);
}

#if (DEBUG_ITIFG & 4)
/* make register dump */
void
vs_dump_regs (itifg_t *iti) {
  
  int count;
  
  for (count = 0; count < VS_REG_SIZE; count++) {
    ITI_PRINT2A("    %s %#04x\n", vs_reg[count],
		vs_read_mm (iti, count, VS_NO_MASK, VS_NO_MIN, VS_NO_MAX));
  }
}
#endif

