/*		 ITI-FG  Kernel module free public release
 *
 *		     Copyright (C) 1996-2000 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: pcdigReg.h,v 7.0 2000/10/25 09:07:58 cvs Exp $
 *
 */

#ifndef _PCDIGREG_H
#define _PCDIGREG_H

#if defined(linux) && defined(__KERNEL__) || \
    defined(__NetBSD__) && defined(_KERNEL) || \
    !defined(OS_DEFINED) || defined(__CONFIG__)

/*
 * Includes:
 */

#include "icpciReg.h"	/* register definitions for the CR_ definitions */
#include "amdigReg.h"	/* register definitions for the DG_ definitions */

/* PC-DIG Hardware Reference Manual; Imaging Technology Inc. */
/* Document Number 402-00002-00; draft 0 February 1999 */

/* R-O   Read-Only Register */
/* W-O   Write-Only Register */
/* R/W   Read/Write Register */
/* R/W1C Clear on Write One to Register */
/* R/W0C Clear on Write Zero to Register */

/* PC-DIG Control Registers (32bit access only), pages 3-26 to 3-82 */
/* **************************************************************** */

#ifdef __cplusplus
extern "C" {
#endif

#define PCD_BRDCFG_MASK		0xE0
#define PCD_BRDCFG_SHIFT	5

#define PCD_BRDCFG_4MB_RS422	0
#define PCD_BRDCFG_4MB_LVDS	1

#define PCD_CR_REG_SIZE		0x400000	/* 4 Mb of register space */
#define PCD_CR_BANK_SIZE	0x080000	/* every bank has 512k space */

#define PCD_RES_INIT		0UL	/* reset reserved bits to zero */

/* 1.    0k : Bus Master Control / Interrupt Control */
/* 2.  512k : Camera Control / Acquisition Control */
/* 3. 1024k : Output Control Table (source scatter gather) 128k x 16 */
/* 4. 1536k : DMA Access Table (destination scatter gather) 64k x 32 */
/* 5. 2048k : Input LUT 0 64k x 16 */
/* 6. 2560k : Input LUT 1 64k x 16 */
/* 7. 3072k : PC16552D Dual UART */
/* 8. 3584k : FPGA Download Control */

#define PCD_DISABLE		CR_DISABLE
#define PCD_ENABLE		CR_ENABLE

/* 1.    0k : Bus Master Control / Interrupt Control */
/*****************************************************/
/* 0x00 0000 Bus Master Control (BMCON) */
typedef union
{
  u_long all;
  struct
  {
    u_long bmenb	:1;	/* Bus Master Enable R/W */
    u_long bmstep	:2;	/* Bus Master Address Step R/W */
#define PCD_BMSTEP_1DDWORD	0	/* increment 1 double word (32bit) */
#define PCD_BMSTEP_2DDWORD	1	/* increment 2 double words (64bit) */
#define PCD_BMSTEP_4DDWORD	2	/* increment 4 double words (128bit) */
    u_long		:29;
  } b;
} pcd_bi_bmcon_t;

/* 0x00 0004 Bus Master LUT Control (BMLUT) R/W */
typedef union
{
  u_long all;
  struct
  {
    u_long octacc	:1;	/* Output Control Table Access R/W */
    u_long dmaacc	:1;	/* DMA Table Access R/W */
    u_long		:30;
  } b;
} pcd_bi_bmlut_t;

/* 0x00 0008 Memory Initialization (SIGINIT) W-O */

/* 0x00 000C Acquire Start Address (AQSTART) R/W */

#define PCD_AQSTART_MASK	0x000003FF
#define PCD_AQSTART_SHIFT	12
#define PCD_AQSTART_INIT	CR_AQSTART_INIT

/* 0x00 0010 Acquire Address Status (AQADR) R-O */

#define PCD_AQADR_MASK		0x00003FFF		
#define PCD_AQADR_SHIFT		8

/* 0x00 0014 Acquire Address Interrupt (INTADR) R/W */

#define PCD_INTADR_MASK		0x0000003F
#define PCD_INTADR_SHIFT	16
#define PCD_INTADR_INIT		CR_INTADR_INIT

/* 0x00 0018 Zoom In Control (ZOOM) */
typedef union
{
  u_long all;
  struct
  {
    u_long		:2;
    u_long octswap	:1;	/* Output Control Table Swap R/W */
#define PCD_OCT_BYTE_SWAP	0	/* bytes are swapped during BMA */
#define PCD_OCT_WORD_SWAP	1	/* wordes are swapped during BMA */
    u_long pad		:2;	/* Pad Mode Enable R/W */
#define PCD_PAD_MODE_SHIFT	2	/* right shift 8 bits to 16 bits */
#define PCD_PAD_MODE_PAD16	3	/* pad higher byte to 16 bit */
    u_long clip		:1;	/* Clip MOde Enable R/W */
    u_long		:26;
  } b;
} pcd_bi_zoom_t;

/* 0x00 001C Segment Size (SEGSIZE) R/W */
#define PCD_SEGSIZE_MASK	0x00003FFF
#define PCD_SEGSIZE_MIN		1
#define PCD_SEGSIZE_MAX		(PCD_SEGSIZE_MASK + PCD_SEGSIZE_MIN)
#define PCD_SEGSIZE_INIT	1UL	/* Process 1 DMA Table Entry */

/* 0x00 0020 OCT Start Address (OCSTART) R/W */
#define PCD_OCTSTART_MASK	0x0000FFFF
#define PCD_OCTSTART_INIT	0UL	/* OCT Entry 0 is the first */

/* 0x00 0024 Software Count (SWCOUNT) R/W */
#define PCD_SWCOUNT_MASK	0x003FFFFF
#define PCD_SWCOUNT_BASE	10227	/* counts per ms (1.023 MHz) */
#define PCD_SWCOUNT_INIT	10227	/* 1ms is default value */

/* 0x00 0028 Reserved */ 
/* 0x00 002C Interrupt Control 1 (INTCON1) */
typedef union
{
  u_long all;
  struct
  {
    u_long adrien	:1;	/* Acquire Address Interrupt Enable R/W */
    u_long bmien	:1;	/* Bus Master Interrupt Enable R/W */
    u_long swctien	:1;	/* Software Counter Interrupt Enable R/W */
    u_long		:29;
  } b;
} pcd_bi_intcon1_t;

/* 0x00 0030 Interrupt Status 1 (INTSTAT1) */
typedef union
{
  u_long all;
  struct
  {
    u_long adrint	:1;	/* Acquire Address Interrupt Status R/W1C */
    u_long bmint	:1;	/* Bus Master Interrupt Status R/W1C */
    u_long swctint	:1;	/* Software Counter Interrupt Status R/W1C */
    u_long		:29;
  } b;
} pcd_bi_intstat1_t;
#define PCD_IRQ1_TEST_MASK	0x00000007	/* here are intr. status bits
						   see xcr_test/clear_irq */
#define PCD_IRQ1_TEST_SHIFT	4	/* this moves the bits to the
					   left places see itifgInt.h */

struct pcd_bi_reg_t
{
  pcd_bi_bmcon_t bmcon;
  pcd_bi_bmlut_t bmlut;
  u_long sgint;
  u_long aqstart;
  u_long aqadr;
  u_long intadr;
  pcd_bi_zoom_t zoom;
  u_long segsize;
  u_long octstart;
  u_long swcount;
  u_long reserved_0;
  pcd_bi_intcon1_t intcon1;
  pcd_bi_intstat1_t intstat1;
};

/* 2.  512k : Camera Control / Acquisition Control */
/***************************************************/
/* 0x08 0000 Camera Timing Control (CTCON) */
typedef union
{
  u_long all;
  struct
  {
    u_long lenenb	:1;	/* LEN Input Enable R/W */
    u_long lensync 	:1;	/* LEN Input Synchronisation R/W */
#define PCD_LEN_SYNC		0	/* synchronous LEN sampling */
#define PCD_LEN_ASYNC		1	/* asynchronous LEN sampling */
    u_long lenpol	:1;	/* LEN Input Polarity R/W */
#define PCD_LEN_ACT_LOW		0	/* falling edge begins new line */
#define PCD_LEN_ACT_HIGH	1	/* rising edge begins new line */
    u_long fenenb	:1;	/* FEN Input Enable R/W */
    u_long fenpol	:1;	/* FEN Input Polarity R/W */
#define PCD_FEN_ACT_LOW		0	/* falling edge begins new frame */
#define PCD_FEN_ACT_HIGH	1	/* rising edge begins new frame */
    u_long multenb	:1;	/* MULTI Input Enable R/W */
    u_long multpol	:1;	/* MULTI Input Polarity R/W */
#define PCD_MULTI_ACT_LOW	0	/* MULTI is active low */
#define PCD_MULTI_ACT_HIGH	1	/* MULTI is active high */
    u_long multmd	:3;	/* MULTI Input Mode R/W */
#define PCD_MULTI_FIELD		0	/* Field information at MULTI input */
#define PCD_MULTI_EOL		1	/* End of Line at MULTI input */
#define PCD_MULTI_DATA_VALID	2	/* PCLK (Data) valid */
    u_long pclkpol	:1;	/* PCLK Input Polarity Status R-O */
#define PCD_PCLK_ACT_HIGH	0	/* Rising edge of PCLK defines data */
#define PCD_PCLK_ACT_LOW	1	/* Falling edge of PCLK defines data */
    u_long		:21;
  } b;
} pcd_ca_ctcon_t;
#define PCD_CTCON_INIT		0x0000007D
				/* LEN Input Enable */
				/* LEN Input Synchronous Sample */
				/* Rising Edge Begins New Line */
				/* FEN Input Enable */
				/* Rising Edge Begins New Frame */
				/* MULTI Input Enable */
				/* MULTI Input is Active High */
				/* Field Information at MULTI Input */
				/* PCLK Polarity is determined by the FPGA */

/* 0x08 0004 Output Timing Control (OUTCON) */
typedef union
{
  u_long all;
  struct
  {
    u_long driveext	:1;	/* Multi-Board Timing Output Enable R/W */
    u_long useext	:1;	/* Multi-Board Timing Input Enable R/W */
    u_long restime	:1;	/* Timing Output Retime (to CCLK) R/W */
    u_long respol	:1;	/* Retime Polarity R/W */
#define PCD_RESTIME_ACT_HIGH	0	/* Synchronize to Rising Edge */
#define PCD_RESTIME_ACT_LOW	1	/* Synchronize to Falling Edge */
    u_long cam3enb	:1;	/* Camera 3 Timing Output Enable R/W */
    u_long exsenb	:1;	/* External Sync Output Enable R/W */
    u_long exspol	:1;	/* External Sync Output Polatity R/W */
#define PCD_EXSYNC_ACT_LOW	0	/* EXSYNC goes low when triggered */
#define PCD_EXSYNC_ACT_HIGH	1	/* EXSYNC goes high when triggered */
    u_long exsmd	:3;	/* External Sync Mode R/W */
#define PCD_EXSYNC_FREE_RUN	0	/* Free Running */
#define PCD_EXSYNC_EXT_TRIG0	1	/* External Trigger 0 */
#define PCD_EXSYNC_EXT_TRIG1	2	/* External Trigger 1 */
#define PCD_EXSYNC_SW_TRIG0	3	/* Software Trigger 0 */
#define PCD_EXSYNC_SW_TRIG1	4	/* Software Trigger 1 */
#define PCD_EXSYNC_VB_TRIG	5	/* Vertical Blank Trigger */
    u_long prienb	:1;	/* Programmable Integration Enable R/W */
    u_long pripol	:1;	/* Programmable Integration Polarity R/W */
#define PCD_PRI_ACT_LOW		0	/* PRI asserted low */
#define PCD_PRI_ACT_HIGH	1	/* PRI asserted high */
    u_long cammd	:4;	/* Camera Mode Control R/W */
#define PCD_CAMMD_MIN		0	/* different modes from 0 */
#define PCD_CAMMD_MAX		15	/* up to 15 are supported */
    u_long cclkenb	:1;	/* CCLK Output Disable R/W */
#define PCD_CCLK_ENABLE		0	/* this is different, we can't use */
#define PCD_CCLK_DISABLE	1	/* the normal DISABLE(0) ENABLE(1) */
    u_long cclkpol	:1;	/* CCLK Output Value R/W */
#define PCD_CCLK_STATIC_LOW	0	/* CCLK Static State = 0 (low) */
#define PCD_CCLK_STATIC_HIGH	1	/* CCLK Static State = 1 (high) */
    u_long		:14;
  } b;
} pcd_ca_outcon_t;
#define PCD_OUTCON_INIT		0x00010C68
				/* Multi-Board Timing Output Disable */
				/* Multi-Board Timing Input Disable */
				/* Timing Output Retime Disable */
				/* Synchronize to Falling Edge */
				/* Camera 3 Timing Output Disable */
				/* External Sync Mode Enable */
				/* EXSYNC goes high when triggered */
				/* External Sync Mode is Free Running */
				/* Programmable Integration Enable */
				/* PRI Asserted High */
				/* Camera Mode Control Value is 0 */
				/* CCLK Output to Camera Disable */
				/* CCLK Pin is High */

/* 0x08 0008 External Sync Control (EXSCON) */
typedef union
{
  u_long all;
  struct
  {
    u_long exsclk	:3;	/* External Snyc CLock R/W */
#define PCD_EXSCLK_14_318_M	0	/*  14.318MHz/ 69.840ns */
#define PCD_EXSCLK_7_159_M	1	/*   7.159MHz/139.680ns */
#define PCD_EXSCLK_3_580_M	2	/*   3.580MHz/279.330ns */
#define PCD_EXSCLK_894_875_k	3	/* 894.875kHz/  1.1175us */
#define PCD_EXSCLK_223_719_k	4	/* 223.719kHz/  4.4699us */
#define PCD_EXSCLK_55_930_k	5	/*  55.930kHz/ 17.880us */
#define PCD_EXSCLK_13_982_k	6	/*  13.982kHz/ 71.520us */
#define PCD_EXSCLK_3_496_k	7	/*   3.496kHz/286.000us */
    u_long		:1;
    u_long exssel	:2;	/* External Sync Select R/W */
#define PCD_EXSSEL_ONE_SHOT	0	/* fixed two clock interval */
#define PCD_EXSSEL_STRT_TO_MID	1	/* assert from the start of the
					   EXSYNC counter to the end of the
					   MIDPNT counter */
#define PCD_EXSSEL_MID_TO_STOP	2	/* assert from the end of the
					   MIDPNT counter to the end of the
					   EXSYNC counter */
    u_long prisel	:2;	/* Programmable Integration Select R/W */
#define PCD_PRISEL_STRT_TO_MID	0	/* assert from the start of the
					   EXSYNC counter to the end of the
					   MIDPNT counter */
#define PCD_PRISEL_MID_TO_STOP	1	/* assert from the end of the
					   MIDPNT counter to the end of the
					   EXSYNC counter */
#define PCD_PRISEL_ONE_SHOT	2	/* fixed two clock interval */
    u_long emidpnt	:12;	/* EXSYNC Down Counter Midpoint R/W */
#define PCD_EMIDPNT_MASK	0x000FFF00
#define PCD_EMIDPNT_SHIFT	4
#define PCD_EMIDPNT_MIN		0
#define PCD_EMIDPNT_MAX		(PCD_EMIDPNT_MASK >> PCD_EMIDPNT_SHIFT)
    u_long prihold	:2;	/* Programmable Integration Hold R/W */
#define PCD_PRIHOLD_LEN_STRT	0	/* Extend PRI to the Start of LEN */
#define PCD_PRIHOLD_LEN_STOP	1	/* Extend PRI to the Stop of LEN */
#define PCD_PRIHOLD_NONE	2	/* Do not Extend the PRI Pulse */
    u_long		:10;
  } b;
} pcd_ca_exscon_t;
#define PCD_EXSCON_INIT		0x00200010
				/* EXSCLK Divider is set by EXSTIME Init */
				/* EXSYNC Starts at Begin and Stops at Mid */
				/* PRI Starts at Begin and Stops at Mid */
				/* EMIDPNT Counter is set by EXSTIME Init */
				/* Do not Extend the PRI Pulse */

/* 0x08 000C External Sync Time (EXSTIME) R/W */
#define PCD_EXSTIME_MASK	0x0000FFFF
#define PCD_EXSTIME_MIN		2
#define PCD_EXSTIME_MAX		(PCD_EXSTIME_MASK + PCD_EXSTIME_MIN)

/* 0x08 0010 Acquire Control (ACQCON) */
typedef union
{
  u_long all;
  struct
  {
    u_long stmode	:3;	/* Start Trigger Mode R/W */
    u_long 		:1;
    u_long etmode	:3;	/* End Trigger Mode R/W */
    u_long 		:1;
#define PCD_TMODE_HWT0_SET	2	/* Start/Stop on HWTrig 0 asserted */
#define PCD_TMODE_HWT0_RESET	3	/* Start/Stop on HWTrig 0 deasserted */
#define PCD_TMODE_HWT1_SET	4	/* Start/Stop on HWTrig 1 asserted */
#define PCD_TMODE_HWT1_RESET	5	/* Start/Stop on HWTrig 1 deasserted */
#define PCD_TMODE_SWT0		6	/* Start/Stop on SWTrig 0 asserted */
#define PCD_TMODE_SWT1		7	/* Start/Stop on SWTrig 1 asserted */
    u_long acqmd	:2;	/* Acquire Command R/W */
#define PCD_FREEZE	CR_FREEZE	/* Stop Acquire */
#define PCD_SNAP	CR_SNAP 	/* Single Frame Acquire */
#define PCD_GRAB	CR_GRAB 	/* Continuous Acquire */
    u_long fldsel	:2;	/* Starting Field Select R/W */
#define PCD_NEXT_FIELD	CR_NEXT_FIELD	/* Start Acquisition at Next Field */
#define PCD_EVEN_FIELD	CR_EVEN_FIELD	/* Start Acquisition at Even Field */
#define PCD_ODD_FIELD	CR_ODD_FIELD	/* Start Acquisition at Odd Field */
    u_long fcnt		:4;	/* Frame Count R/W */
#define PCD_FCNT_MIN		1	/* we can acquire from 1 frame... */
#define PCD_FCNT_MAX		8	/* up to 8 frames */
#define PCD_FCNT_INFINITE	15	/* this is acquire without reset */
    u_long newaq	:1;	/* New Acquire Command Status R-O */
    u_long gstat	:1;	/* Grab Status R-O */
    u_long fstrt	:1;	/* Field Start Status R-O */
#define PCD_FIELD_EVEN	CR_FIELD_EVEN	/* I-laced Video start on even field */
#define PCD_FIELD_ODD	CR_FILED_ODD	/* I-laced Video start on odd field */
    u_long 		:13;
  } b;
} pcd_ca_acqcon_t;
 
/* 0x08 0014 FIFO Initialization (FIFOINIT) W-O */
/* 0x08 0018 Horizontal Offset (HOFF) R/W */
#define PCD_HOFF_MASK		0x0000FFFF
#define PCD_HOFF_INIT		DG_HOFF_INIT

/* 0x08 001C Horizontal Active (HACT) R/W */
#define PCD_HACT_MASK		0x0000FFFF
#define PCD_HACT_MIN		1
#define PCD_HACT_MAX		(PCD_HACT_MASK + PCD_HACT_MIN)
#define PCD_HACT_INIT		DG_HACT_INIT

/* 0x08 0020 Vertical Offset (VOFF) R/W */
#define PCD_VOFF_MASK		0x0000FFFF
#define PCD_VOFF_INIT		DG_VOFF_INIT

/* 0x08 0024 Vertical Active (VACT) R/W */
#define PCD_VACT_MASK		0x0000FFFF
#define PCD_VACT_MIN		1
#define PCD_VACT_MAX		(PCD_VACT_MASK + PCD_VACT_MIN)
#define PCD_VACT_INIT		DG_VACT_INIT

/* 0x08 0028 Camera Clock Control (CCLKCON) */
typedef union
{
  u_long all;
  struct
  {
    u_long syndat	:1;	/* Synthesizer Frequency Select Data R/W */
    u_long synclka	:1;	/* Syntheszier A Serial Clock Enable */
    u_long synclkb	:1;	/* Syntheszier B Serial Clock Enable */
    u_long synmuxa	:1;	/* Synthesizer A Mux Select */
    u_long synmuxb	:1;	/* Synthesizer B Mux Select */
#define PCD_SYNMUX_PROG		0	/* Use Programmed Frequency */
#define PCD_SYNMUX_DEF		1	/* Use Default Frequency 14.318 MHz */
    u_long		:27;
  } b;
} pcd_ca_cclkcon_t;
#define PCD_CLKVAL_REFCLOCK	14.31818	/* MHz XTAL OSC */
#define PCD_CLKVAL_SHIFT	22		/* 22bit value */
#define PCD_CLKVAL_MASK		((1 << PCD_CLKVAL_SHIFT) - 1)
#define PCD_CLKVAL_SYSTEM	41.67	/* PCCDig internal master clock */

#define PCD_CCLKFREQ_INIT	PCD_CLKVAL_REFCLOCK

/* 0x08 002C Latched Vertical Active (LVACT) R-O */
#define PCD_LVACT_MASK		0x0000FFFF

/* 0x08 0030 LUT 0 Control (LUTCON0) */
typedef union
{
  u_long all;
  struct
  {
    u_long lut0psz	:3;	/* LUT0 Pixel Size R/W */
#define PCD_PSIZE_8	DG_PSIZE_8	/*  8-bit */
#define PCD_PSIZE_10	DG_PSIZE_10	/* 10-bit */
#define PCD_PSIZE_12	DG_PSIZE_12	/* 12-bit */
#define PCD_PSIZE_14		3	/* 14-bit */
#define PCD_PSIZE_16		4	/* 16-bit */
#define PCD_PSIZE_8_MASK	DG_PSIZE_8_MASK	/* 256 pages available */
#define PCD_PSIZE_8_SHIFT	DG_PSIZE_8_SHIFT /* page size 256 (1 * 256) */
#define PCD_PSIZE_10_MASK	DG_PSIZE_10_MASK /* 64 pages available */
#define PCD_PSIZE_10_SHIFT	DG_PSIZE_10_SHIFT /* page size 1024 (4*256) */
#define PCD_PSIZE_12_MASK	DG_PSIZE_12_MASK /* 16 pages available */
#define PCD_PSIZE_12_SHIFT	DG_PSIZE_12_SHIFT /* page size 4096 (16*256) */
#define PCD_PSIZE_14_MASK	0x03	/* 4 pages available */
#define PCD_PSIZE_14_SHIFT	6	/* page size 16384 (64 * 256) */
#define PCD_PSIZE_16_MASK	DG_PSIZE_16_MASK /* 1 page available */
#define PCD_PSIZE_16_SHIFT	DG_PSIZE_16_SHIFT /* page size 65536(256*256)*/
    u_long lutacc	:1;	/* LUT0/1 Access Control R/W */
    u_long		:4;
    u_long lut0pg	:8;	/* LUT0 Page Select R/W */
    u_long		:16;
  } b;
} pcd_ca_lutcon0_t;
#define PCD_LUTCON0_INIT	0x00000000
				/* LUT configuration is 8bit/256pages */
				/* Disable host access */
				/* Select LUT page 0 */

/* 0x08 0034 LUT 1 Control (LUTCON1) */
typedef union
{
  u_long all;
  struct
  {
    u_long lut1psz	:3;	/* LUT1 Pixel Size R/W */
    u_long		:5;
    u_long lut1pg	:8;	/* LUT1 Page Select R/W */
    u_long		:16;
  } b;
} pcd_ca_lutcon1_t;
#define PCD_LUTCON1_INIT	0x00000000
				/* LUT configuration is 8bit/256pages */
				/* Disable host access */
				/* Select LUT page 0 */

/* 0x08 0038 Trigger Control (TRIGCON) */
typedef union
{
  u_long all;
  struct
  {
    u_long		:1;
    u_long trig0pol	:1;	/* Trigger 0 Input Polarity R/W */
#define PCD_TRIG_ACT_LOW	0	/* Falling Edge/Low Level Triggers */
#define PCD_TRIG_ACT_HIGH	1	/* Rising Edge/High Level Triggers */
    u_long trig0src	:2;	/* Trigger 0 Source Select R/W */
#define PCD_TRIG_SRC_DIFF	0	/* Differential Trigger Input */
#define PCD_TRIG_SRC_TTL	1	/* TTL Trigger Input */
#define PCD_TRIG_SRC_OPTO	2	/* Opto-Coupled Trigger Input */
    u_long trig0gd	:1;	/* Trigger 0 Glitch Detector R/W */
    u_long		:4;
    u_long trig1pol	:1;	/* Trigger 1 Input Polarity R/W */
    u_long trig1src	:2;	/* Trigger 1 Source Select R/W */
    u_long trig1gd	:1;	/* Trigger 1 Glitch Detector R/W */
    u_long 		:19;
  } b;
} pcd_ca_trigcon_t;
#define PCD_TRIGCON_INIT	0x00008080
				/* Falling Edge/Low Level Causes Trigger 0/1 */
				/* Opto-Coupled Trigger 0/1 Input */
				/* Disable Trigger 0/1 Glitch Detector */
	
/* 0x08 003C Trigger Status (TRIGSTAT) */
typedef union
{
  u_long all;
  struct
  {
    u_long trig0diff	:1;	/* Trigger 0 Differential Input R-O */
    u_long trig0ttl	:1;	/* Trigger 0 TTL Input R-O */
    u_long trig0opto	:1;	/* Trigger 0 Opto-Coupled Input R-O */
    u_long trig0cyc	:1;	/* Trigger 0 Cycle Status R-O */
#define PCD_OUT_TRIGCY		0	/* No Trigger Cycle in Progress */
#define PCD_IN_TRIGCY		1	/* Trigger Cycle in Progress */
    u_long trig1diff	:1;	/* Trigger 1 Differential Input R-O */
    u_long trig1ttl	:1;	/* Trigger 1 TTL Input R-O */
    u_long trig1opto	:1;	/* Trigger 1 Opto-Coupled Input R-O */
    u_long trig1cyc	:1;	/* Trigger 1 Cycle Status R-O */
    u_long camlen	:1;	/* Camera LEN Input Status R-O */
    u_long camfen	:1;	/* Camera FEN Input Status R-O */
    u_long cammult	:1;	/* Camera MULTI Input Status R-O */
    u_long		:21;
  } b;
} pcd_ca_trigstat_t;

/* 0x08 0040 Software Trigger 0 (SWTRIG0) W-O */
/* 0x08 0044 Software Trigger 1 (SWTRIG1) W-O */
/* 0x08 0048 Interrupt Control 0 (INTCON0) */
typedef union
{
  u_long all;
  struct
  {
    u_long vbien	:1;	/* Vertical Blank Interrupt Enable R/W */
    u_long hbien	:1;	/* Horizontal Blank Interrupt Enable R/W */
    u_long trig0ien	:1;	/* Trigger 0 Interrupt Enable R/W */
    u_long trig1ien	:1;	/* Trigger 1 Interrupt Enable R/W */
    u_long soaqien	:1;	/* Start of Acquire Interrupt Enable R/W */
    u_long eoaqien	:1;	/* End of Acquire Interrupt Enable R/W */
    u_long pioien	:1;	/* Parallel I/O Port Interrupt Enable R/W */
    u_long		:25;
  } b;
} pcd_ca_intcon0_t;

/* 0x08 004C Interrupt Status 0 (INTSTAT0) */
typedef union
{
  u_long all;
  struct
  {
    u_long vbint	:1;	/* Vertical Blank Interrupt Status R/W1C */
    u_long hbint	:1;	/* Horizontal Blank Interrupt Status R/W1C */
    u_long trig0int	:1;	/* Trigger 0 Interrupt Status R/W1C */
    u_long trig1int	:1;	/* Trigger 1 Interrupt Status R/W1C */
    u_long soaqint	:1;	/* Start of Acquire Interrupt Status R/W1C */
    u_long eoaqint	:1;	/* End of Acquire Interrupt Status R/W1C */
    u_long pioint	:1;	/* Parallel I/O Port Interrupt Status R/W1C */
    u_long		:25;
  } b;
} pcd_ca_intstat0_t;
#define PCD_IRQ0_TEST_MASK	0x0000007F	/* here are intr. status bits
						   see xcr_test/clear_irq */
#define PCD_IRQ0_TEST_SHIFT	24	/* this moves the bits to the
					   left places see itifgInt.h */


/* 0x08 0050 Trigger Divider (TRIGDIV) */
typedef union
{
  u_long all;
  struct
  {
    u_long trigcnt	:8;	/* Trigger Count Down Status R-O */
    u_long trigdiv	:8;	/* Trigger Divide Factor R/W */
    u_long divcon	:2;	/* Divide Control R/W */
#define PCD_DIVCON_CLEAR	0	/* Disable and Clear Counter */
#define PCD_DIVCON_TRIG0	2	/* Divide Hardware Trigger 0 */
#define PCD_DIVCON_TRIG1	3	/* Divide Hardware Trigger 1 */
    u_long 		:14;
  }b;
} pcd_ca_trigdiv_t;
#define PCD_TRIGDIV_INIT	0x00000000
				/* Divide Harware Trigger by 1 */
				/* Disable and Clear Counter */

/* 0x08 0054 Present Vertical Active (PVACT) R-O */
#define PCD_PVACT_MASK		0x0000FFFF

/* 0x00 0058 PRI Midpoint (PMIDPNT) R/W */

#define PCD_PMIDPNT_MASK	0x00000FFF
#define PCD_PMIDPNT_SHIFT	4
/* This Register is Initialized by EXSTIME Init */

/* 0x08 005C Decimation Control (DECIM) */
typedef union
{
  u_long all;
  struct
  {
    u_long hdec		:3;	/* Horizontal Decimation R/W */
    u_long vdec		:3;	/* Vertical Decimation R/W */
#define PCD_DEC_BY_1	CR_DEC_BY_1	/* no decimation (original image) */
#define PCD_DEC_BY_2	CR_DEC_BY_2	/* a half of the width/height */
#define PCD_DEC_BY_4	CR_DEC_BY_4	/* a quarter of the width/height */
#define PCD_DEC_BY_8	CR_DEC_BY_8	/* 1/8 of the width/height */
#define PCD_DEC_BY_16	CR_DEC_BY_16	/* 1/16 of the width/height */
    u_long 		:26;
  } b;
} pcd_ca_decim_t;

/* 0x08 0060 Storage Format (SFORM) R-O */
typedef union
{
  u_long all;
  struct
  {
    u_long sform	:3;
#define PCD_SFORM_8BIT_ST	0	/* 8bit single tap */
#define PCD_SFORM_16BIT_ST	1	/* 10,12,14,16bit single tap */
#define PCD_SFORM_24BIT_PST	2	/* 24bit packed single tap */
#define PCD_SFORM_32BIT_ST	3	/* 32bit single tap */
#define PCD_SFORM_8BIT_DT	4	/* 8bit dual tap */
#define PCD_SFORM_8BIT_QT	5	/* 8bit quad tap */
#define PCD_SFORM_16BIT_DT	6	/* 10,12,14,16bit dual dual tap */
    u_long		:29;
  } b;
} pcd_ca_sform_t;

/* 0x08 0064 Reserved */
/* 0x08 0068 Port Control (PORTCON) */
typedef union
{
  u_long all;
  struct
  {
    u_long outstrb	:1;	/* Output Port Strobe R/W */
    u_long inregenb	:1;	/* Input Buffer Enable R/W */
    u_long instrbpol	:1;	/* Input Strobe Polarity R/W */
#define PCD_INSTRB_ACT_HIGH	0	/* Rising Edge Clocks Data into Buf */
#define PCD_INSTRB_ACT_LOW	1	/* Falling Edge Clocks Data into Buf */
    u_long inregclr	:1;	/* Input Buffer Clear R/W */
    u_long inpintpol	:1;	/* Input Port Interrupt Polarity R/W */
#define PCD_INPINT_ACT_HIGH	0	/* Rising Edge Interrupt */
#define PCD_INPINT_ACT_LOW	1	/* Falling Edge Interrupt */
    u_long inprint	:1;	/* Input Port Interrupt Status R-O */
    u_long inprstrb	:1;	/* Input Port Strobe Pin Status R-O */
    u_long		:25;
  } b;
} pcd_ca_portcon_t;

/* 0x08 006C Output Port (OUTPORT) R/W */
#define PCD_OUTPORT_MASK	0x000000FF
#define PCD_OUTPORT_INIT	0xFF	/* drive all outputs to high */

/* 0x08 0070 Input Port (INPORT) R-O */
#define PCD_INPORT_MASK		0x000000FF

struct pcd_ca_reg_t
{
  pcd_ca_ctcon_t ctcon;
  pcd_ca_outcon_t outcon;
  pcd_ca_exscon_t exscon;
  u_long exstime;
  pcd_ca_acqcon_t acqcon;
  u_long fifoinit;
  u_long hoff;
  u_long hact;
  u_long voff;
  u_long vact;
  pcd_ca_cclkcon_t cclkcon;
  u_long lvact;
  pcd_ca_lutcon0_t lutcon0;
  pcd_ca_lutcon1_t lutcon1;
  pcd_ca_trigcon_t trigcon;
  pcd_ca_trigstat_t trigstat;
  u_long swtrig0;
  u_long swtrig1;
  pcd_ca_intcon0_t intcon0;
  pcd_ca_intstat0_t intstat0;
  pcd_ca_trigdiv_t trigdiv;
  u_long pvact;
  u_long pmidpnt;
  pcd_ca_decim_t decim;
  pcd_ca_sform_t sform;
  u_long reserved_2;
  pcd_ca_portcon_t portcon;
  u_long outport;
  u_long inport;
};

/* 3. 1024k : Output Control Table (source scatter gather) 128k x 16 */
/*********************************************************************/
/* 0x10 0000 - 0x17 FFFC */
#define PCD_NUM_OCT_ENTRYS	(1 << 16)
typedef union
{
  u_long all;
  struct
  {
    u_long addr		:16;	/* Low Part Transfer Start Address R/W */
#define PCD_OCT_LOW_ADDR_MASK	0x0000FFFF
#define PCD_OCT_LOW_ADDR_SHIFT	3
    u_long		:16;
  } b;
} pcd_octdata_low_t;

typedef union
{
  u_long all;
  struct
  {
    u_long addr		:4;	/* High Part Transfer Start Address R/W */
#define PCD_OCT_HIGH_ADDR_SHIFT	19
    u_long swap		:1;	/* Transfer Swap Enable R/W */
    u_long decr		:1;	/* Decrement Address R/W */
    u_long last		:1;	/* this is the last block to transfer */
    u_long xcnt		:5;	/* Transfer Count R/W */
#define PCD_OCT_SPLIT_SIZE	256	/* the maximal transfer count */
#define PCD_OCT_SPLIT_SHIFT	3
    u_long		:20;
  } b;
} pcd_octdata_high_t;

typedef struct
{
  pcd_octdata_low_t low;
  pcd_octdata_high_t high;
} pcd_octdata_t;

enum pcd_geo_t
{
  PCD_1CHAN_L2R,
  PCD_2CHAN_INTERLEAVED,
  PCD_2CHAN_SEP_TAP_L2R,
  PCD_2CHAN_SEP_TAP_R2L,
  PCD_4CHAN_SEP_TAP_L2R,
  PCD_4CHAN_SEP_TAP_R2L,
  PCD_4CHAN_2SEG_INTERLEAVED,
  PCD_4CHAN_INTERLEAVE_CONVERGE,
  PCD_2CHAN_SEP_TAP_CONVERGE,
  PCD_SEP_QUAD_TAP_CONVERGE,
  PCD_2TAP_INTERLINE_A_EVEN,
  PCD_2TAP_INTERLINE_B_EVEN
};

/* 4. 1536k : DMA Access Table (destination scatter gather) 64k x 32 */
/********************************************************************/
/* 0x18 0000 - 0x1B FFFC */
#define PCD_NUM_DMA_ENTRYS	(1 << 15)
#define PCD_INIT_AGCSTS		0x10000000
#define PCD_INIT_AINT		0x00244000
typedef struct
{
  u_long addr;	/* 32bit destination start address (aligned) */
#define PCD_DMA_TABLE_MOD	4
  u_long xcnt;
#define PCD_DMA_XCNT_MASK	0x03FFFFFF
} pcd_dmadata_t;

/* 5. 2048k : Input LUT 0 64k x 16 */
/***********************************/
/* 0x20 0000 - 0x23 FFFC */
#define PCD_NUM_LUT_ENTRYS	(1 << 16)

/* 6. 2560k : Input LUT 1 64k x 16 */
/***********************************/
/* 0x28 0000 - 0x2B FFFC */

/* 7. 3072k : PC16552D Dual UART */
/*********************************/
/* 0x30 0000 - 0x30 001C Channel 2 */
/* 0x30 0020 - 0x30 003C Channel 1 */

/* 8. 3584k : FPGA Download Control */
/************************************/
/* 0x38 0000 FPGA Control (FPGACON) */
typedef union
{
  u_long all;
  struct
  {
    u_long fpgaprg	:1;	/* FPGA Program Control R/W */
    u_long fpgardy	:1;	/* FPGA Ready R-O */
    u_long fpgadone	:1;	/* FPGA Done R-O */
    u_long brdid	:3;	/* Board Version ID R-O */
    u_long		:26;
  } b;
} pcd_fp_fpgacon_t;

/* 0x38 003C FPGA Configuration Data W-O */
struct pcd_fp_reg_t
{
  pcd_fp_fpgacon_t fpgacon;
  u_long reserved_0;
  u_long reserved_1;
  u_long reserved_2;
  u_long reserved_3;
  u_long reserved_4;
  u_long reserved_5;
  u_long reserved_6;
  u_long reserved_7;
  u_long reserved_8;
  u_long reserved_9;
  u_long reserved_A;
  u_long reserved_B;
  u_long reserved_C;
  u_long reserved_D;
  u_long fpgadata;
};

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* linux && __KERNEL__ || !OS_DEFINED || defined(__CONFIG__) */

#endif /* _PCDIGREG_H */

